"""OpenGL text utilities

This module contains helper utilities for rendering text under
pygext.gl
"""

from OpenGL.GL import *
import pygame
from pygame.locals import Rect

from pygext.gl.shapes import Bitmap
from pygext.gl.shapes.base import GLShape
from pygext.color import _conv_color


try:
    from Polygon import Polygon
except ImportError:
    pass

__all__ = [
    'GLFont',
    ]

class GLFont(object):
    """OpenGL wrapper for Pygame fonts
    
    If you have a lot of text in your application, this class will help
    you conserve texture memory by pre-generating all visible characters
    in a font and reusing the same textures in each GLText shape.
    """
    
    def __init__(self, font, color=(255,255,255), antialias=True, spacing=0):
        """Create a new GLFont wrapper around a Pygame Font.

        font       - a pygame font object to be converted into an opengl font
	           OR a (fontname, fontsize) tuple
        color      - color of the rendered font (default white)
        antialias  - antialias flag (default True)
	
	Creating a font by its name will first try to load the font from current
	working directory and if that fails, it looks for the font in system fonts.
	
	NOTE: The color parameter should be considered deprecated at this point.
	It's better to leave the font color white, and change the color of individual
	TextEntities instead.
        """
        if isinstance(font, tuple):
            fontname, size = font
            try:
                f = pygame.font.Font(fontname, size)
            except IOError:
                f = pygame.font.SysFont(fontname, size)
            font = f
        self.font = font
        self.color = color
        self.antialias = antialias
        self.spacing = spacing
        self.chardict = {}
        self._generate_surface()

    def render(self, text, color=None):
        """Generate a text entity using the font.

        text  - string containing the text
        color - optional color argument (only works if no color was defined for the GLFont object)
	@return GLText shape rendered using the font
        """
        return GLText(self, text, color)


    def _generate_surface(self, chr_range=range(32,128)):
        """
        Internal helper function, that creates a Bitmap entity from each
        separate letter of the font.

        XXX TODO: the letters should be packed more intelligently instead of
        creating a separate texture from each character.
        """
        for c in map(chr, chr_range):
            bmp = self.font.render(c+" ", self.antialias, self.color)
            w,h = self.font.size(c)
            self.chardict[c] = Bitmap(bmp.subsurface(0,0,w+self.spacing,h), hotspot=(0,1))

class GLText(GLShape):
    """
    Shape object that represents text.
    
    GLText objects should not usually be created directly,
    but instead via the GLFont.render method.
    
    For short, throwaway texts, you can also contruct a GLText
    object directly from a pygame Font object by calling:
    
    GLFont(font, "Foobar", color=(255,0,0))
    """

    def init(self, glfont, text, color=None):
	"""intializer that is called by GLShape, do not use directly"""
        if type(glfont) is pygame.font.Font:
            from pygext.gl.shapes.bitmap import Bitmap
            s = glfont.render(text, True, color or (255,255,255))
            self.__class__ = Bitmap ## XXX: is this too hoaxy?
            self.init(s)
            return
        self.glfont = glfont
        self.text = text
        self.rect = None
        self.color = color

    def _execute(self):
        glPushMatrix()
        color = self.color
        if color is not None:
            oldcol = glGetFloatv(GL_CURRENT_COLOR)
            glColor4f(*_conv_color(*color))

        xpos = 0
        ypos = 0
        xmax = 0
        height = self.glfont.font.get_linesize()
        glTranslatef(0, height,0)
        for c in self.text:
            if c == '\n':
                xmax = max(xpos, xmax)
                ypos += height
                glTranslatef(-xpos, height, 0)
                xpos = 0
            bmp = self.glfont.chardict.get(c, None)
            if bmp is None:
                continue
            bmp._execute()
            x,y = bmp.size
            glTranslatef(x,0,0)
            xpos += x
        glPopMatrix()
        if color is not None:
            glColor4f(*oldcol)            
        xmax = max(xpos, xmax, 1)
        self.rect = Rect(0,0, xmax, ypos+height)

    def get_stencil_poly(self):
        r = self.rect
        p = Polygon( [ (0,0), (r.right, 0), (r.right, r.bottom), (0, r.bottom) ] )
        return self.trans.transform_polygon(p)

    def get_stencil_rect(self):
        return self.trans.transform_rect(self.rect)
